#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Pilou24Positions
#      By:	sdmitch
#   Usage:	display the twenty four possible position of a cube.
#	 Note:	
#			
#    Date:	Jun 2011
#------------------------------------------------------------------------------------------------
require 'Sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Twenty Four Positions') { Pilou.twenty_four_positions }
end
#
file_loaded(filename)
#
module Pilou

	def Pilou.twenty_four_positions
		mod=Sketchup.active_model
		ent=mod.entities
		sel=mod.selection
		if ((sel.first.class == Sketchup::Group || sel.first.class == Sketchup::ComponentInstance) && sel.length == 1)
			mod.start_operation "Twenty Four Positions"
			grps=[]; grps.push sel.first
			x_axis = [1,0,0]; y_axis = [0,1,0]; z_axis = [0,0,1]
			if grps[0].class == Sketchup::Group
				dx = grps[0].local_bounds.max.x - grps[0].local_bounds.min.x
				dy = grps[0].local_bounds.max.y - grps[0].local_bounds.min.y
				for n in 1..23
					grps[n]=grps[0].copy
				end
			else
				dx = grps[0].bounds.max.x - grps[0].bounds.min.x
				dy = grps[0].bounds.max.y - grps[0].bounds.min.y
				pt = grps[0].transformation.origin
				tr = Geom::Transformation.new(pt)
				grps[0].name="Source";
				for n in 1..23
					grps[n]=ent.add_instance(grps[0].definition,tr)
					grps[n].make_unique
				end
			end
			ans =UI.inputbox(["Gap:"],[(dx/10).to_l],"Spacing between cubes")
			if !ans then;return;end;dx += ans[0]; dy += ans[0]
			irow = 0; igrp = 0; # First Row
			for icol in 1..3
				Pilou.rotate(grps[icol],z_axis,icol*90.degrees)
				Pilou.translate(grps[icol],dx*icol,dy*irow,0)
				grps[icol]=self.explode_and_remake(grps[icol])
				grps[icol].name="Source"
			end
			irow = 1; igrp = 4; # Second Row
			Pilou.rotate(grps[igrp],x_axis,irow*90.degrees)
			Pilou.translate(grps[igrp],0,irow*dy,0)
			grps[igrp]=self.explode_and_remake(grps[igrp])
			grps[igrp].name="Source"
			for icol in 1..3
				Pilou.rotate(grps[icol+igrp],x_axis,irow*90.degrees)
				Pilou.rotate(grps[icol+igrp],z_axis,icol*90.degrees)
				Pilou.translate(grps[icol+igrp],dx*icol,dy*irow,0)
				grps[icol+igrp]=self.explode_and_remake(grps[icol+igrp])
				grps[icol+igrp].name="Source"
			end
			irow = 2; igrp = 8; # Third Row
			Pilou.rotate(grps[igrp],x_axis,irow*90.degrees)
			Pilou.translate(grps[igrp],0,irow*dy,0)
			grps[igrp]=self.explode_and_remake(grps[igrp])
			grps[igrp].name="Source"
			for icol in 1..3
				Pilou.rotate(grps[icol+igrp],x_axis,irow*90.degrees)
				Pilou.rotate(grps[icol+igrp],z_axis,icol*90.degrees)
				Pilou.translate(grps[icol+igrp],dx*icol,dy*irow,0)
				grps[icol+igrp]=self.explode_and_remake(grps[icol+igrp])
				grps[icol+igrp].name="Source"
			end
			irow = 3; igrp = 12; # Forth Row
			Pilou.rotate(grps[igrp],x_axis,irow*90.degrees)
			Pilou.translate(grps[igrp],0,irow*dy,0)
			grps[igrp]=self.explode_and_remake(grps[igrp])
			grps[igrp].name="Source"
			for icol in 1..3
				Pilou.rotate(grps[icol+igrp],x_axis,irow*90.degrees)
				Pilou.rotate(grps[icol+igrp],z_axis,icol*90.degrees)
				Pilou.translate(grps[icol+igrp],dx*icol,dy*irow,0)
				grps[icol+igrp]=self.explode_and_remake(grps[icol+igrp])
				grps[icol+igrp].name="Source"
			end
			irow = 4; igrp = 16; # Fifth Row
			rotate(grps[igrp],y_axis,90.degrees)
			translate(grps[igrp],0,irow*dy,0)
			grps[igrp]=self.explode_and_remake(grps[igrp])
			grps[igrp].name="Source"
			for icol in 1..3
				Pilou.rotate(grps[icol+igrp],y_axis,90.degrees)
				Pilou.rotate(grps[icol+igrp],z_axis,icol*90.degrees)
				Pilou.translate(grps[icol+igrp],dx*icol,dy*irow,0)
				grps[icol+igrp]=self.explode_and_remake(grps[icol+igrp])
				grps[icol+igrp].name="Source"
			end
			irow = 5; igrp = 20; # Sixth Row
			rotate(grps[igrp],y_axis,270.degrees)
			translate(grps[igrp],0,irow*dy,0)
			grps[igrp]=self.explode_and_remake(grps[igrp])
			grps[igrp].name="Source"
			for icol in 1..3
				Pilou.rotate(grps[icol+igrp],y_axis,270.degrees)
				Pilou.rotate(grps[icol+igrp],z_axis,icol*90.degrees)
				Pilou.translate(grps[icol+igrp],dx*icol,dy*irow,0)
				grps[icol+igrp]=self.explode_and_remake(grps[icol+igrp])
				grps[icol+igrp].name="Source"
			end
			mod.commit_operation
		else
			UI.messagebox "Select a single group or component"
		end
	end

	def Pilou.translate(gorc,x,y,z)
		tr=Geom::Transformation.translation([x,y,z])
		gorc.transform! tr
	end

	def Pilou.rotate(gorc,ax,ang)
		pt = gorc.local_bounds.center if gorc.class == Sketchup::Group
		pt = gorc.bounds.center if gorc.class == Sketchup::ComponentInstance
		tr=Geom::Transformation.rotation(pt,ax,ang)
		gorc.transform! tr
	end
	
	def self.explode_and_remake(gorc)
		mod=Sketchup.active_model
		ent=mod.entities
		sel=mod.selection
		dump=gorc.explode; sel.clear
		dump.each{|e| sel.add e if (e.is_a?(Sketchup::Face) || e.is_a?(Sketchup::Edge))}
		grp=ent.add_group sel
		gorc=grp.to_component
	end
end

